// Libraries
import { Component, OnInit, Inject } from '@angular/core';
import { MatDialogRef, MAT_DIALOG_DATA } from '@angular/material';
import { FormGroup, FormControl, Validators } from '@angular/forms';
import { Store } from '@ngrx/store';

// State management
import { GlobalState } from '@app/core/state';
import {
  WriteNote,
  DeleteNote,
} from '@app/degree-planner/store/actions/note.actions';

export type NotesDialogData =
  | { termCode: string; hasExistingNote: true; initialText: string }
  | { termCode: string; hasExistingNote: false; initialText?: never };

@Component({
  selector: 'cse-notes-dialog',
  templateUrl: './notes-dialog.component.html',
  styleUrls: ['./notes-dialog.component.scss'],
})
export class NotesDialogComponent implements OnInit {
  public form: FormGroup;
  public MAX_LENGTH = 512;

  constructor(
    private store: Store<GlobalState>,
    private dialogRef: MatDialogRef<NotesDialogComponent>,
    @Inject(MAT_DIALOG_DATA) public data: NotesDialogData,
  ) {}

  public ngOnInit() {
    this.form = new FormGroup({
      textarea: new FormControl(this.data.initialText || '', [
        Validators.maxLength(this.MAX_LENGTH),
        Validators.required,
      ]),
    });
  }

  public writeNote() {
    const termCode = this.data.termCode;
    const noteText =
      typeof this.form.value.textarea === 'string'
        ? this.form.value.textarea
        : '';
    this.store.dispatch(new WriteNote({ termCode, noteText }));
    this.dialogRef.close({ event: 'save' });
  }

  public deleteNote() {
    const termCode = this.data.termCode;
    this.store.dispatch(new DeleteNote({ termCode }));
    this.dialogRef.close({ event: 'remove' });
  }
}
