import { Action } from '@ngrx/store';
import { Course } from '@app/core/models/course';

export enum CourseActionTypes {
  AddCourse = '[Course] Add',
  AddCourseSuccess = '[Course] Add (Success)',

  RemoveCourse = '[Course] Remove',
  RemoveCourseSuccess = '[Course] Remove (Success)',

  MoveCourseBetweenTerms = '[Course] Move Between Terms',
  MoveCourseBetweenTermsSuccess = '[Course] Move Between Terms (Success)',

  AddSaveForLater = '[Course] Add Save for Later',
  AddSaveForLaterSuccess = '[Course] Add Save for Later (Success)',

  RemoveSaveForLater = '[Course] Remove Save for Later',
  RemoveSaveForLaterSuccess = '[Course] Remove Save for Later (Success)',

  MoveSaveForLaterToTerm = '[Course] Save for Later to Term',
  MoveSaveForLaterToTermSuccess = '[Course] Save for Later to Term (Success)',

  CourseError = '[Course] Error',
}

export class MoveCourseBetweenTerms implements Action {
  public readonly type = CourseActionTypes.MoveCourseBetweenTerms;
  constructor(public payload: { to: string; from: string; id: number }) {}
}

export class MoveCourseBetweenTermsSuccess implements Action {
  public readonly type = CourseActionTypes.MoveCourseBetweenTermsSuccess;
  constructor(public payload: { to: string; from: string; id: number }) {}
}

export class AddCourse implements Action {
  public readonly type = CourseActionTypes.AddCourse;
  constructor(
    public payload: { subjectCode: string; courseId: string; termCode: string },
  ) {}
}

export class AddCourseSuccess implements Action {
  public readonly type = CourseActionTypes.AddCourseSuccess;
  constructor(public payload: { course: Course }) {}
}

export class RemoveCourse implements Action {
  public readonly type = CourseActionTypes.RemoveCourse;
  constructor(public payload: { recordId: number }) {}
}

export class RemoveCourseSuccess implements Action {
  public readonly type = CourseActionTypes.RemoveCourseSuccess;
  constructor(public payload: { recordId: number }) {}
}

export class AddSaveForLater implements Action {
  public readonly type = CourseActionTypes.AddSaveForLater;
  constructor(
    public payload: {
      subjectCode: string;
      courseId: string;
      title: string;
      catalogNumber: string;
    },
  ) {}
}

export class AddSaveForLaterSuccess implements Action {
  public readonly type = CourseActionTypes.AddSaveForLaterSuccess;
  constructor(
    public payload: {
      subjectCode: string;
      courseId: string;
      title: string;
      catalogNumber: string;
    },
  ) {}
}

export class RemoveSaveForLater implements Action {
  public readonly type = CourseActionTypes.RemoveSaveForLater;
  constructor(public payload: { subjectCode: string; courseId: string }) {}
}

export class RemoveSaveForLaterSuccess implements Action {
  public readonly type = CourseActionTypes.RemoveSaveForLaterSuccess;
  constructor(public payload: { subjectCode: string; courseId: string }) {}
}

export class CourseError implements Action {
  public readonly type = CourseActionTypes.CourseError;
  constructor(public payload: { message: string; error: any }) {}
}
